﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;

namespace Umc.Core
{
	/// <summary>
	/// Umc.Core<br/>
	/// Delegate 를 담는 EventHandlerDictionary 컬렉션
	/// </summary>
	public class EventHandlerDictionary : IDisposable
	{
		/// <summary>
		/// Umc.Core<br/>
		/// Event 를 담는 컬렉션
		/// </summary>
		private Dictionary<object,Delegate> eventDictionary	= new Dictionary<object,Delegate>();

		/// <summary>
		/// Umc.Core<br/>
		/// EventHandlerDictionary 에 EventHandler 를 추가한다.
		/// </summary>
		/// <param name="key">키값</param>
		/// <param name="value">Delegate</param>
		public void AddHandler(object key, Delegate value)
		{
			if ( eventDictionary.ContainsKey(key) )
			{
				eventDictionary[key] = Delegate.Combine(eventDictionary[key], value);
			}
			else
			{
				eventDictionary[key] = value;
			}
		} 

		/// <summary>
		/// Umc.Core<br/>
		/// EventHandlerDictionary 에 EventHandler 를 제거한다.
		/// </summary>
		/// <param name="key">키값</param>
		/// <param name="value">Delegate</param>
		public void RemoveHandler(object key, Delegate value)
		{
			if ( eventDictionary.ContainsKey(key) )
			{
				eventDictionary[key] = Delegate.Remove(eventDictionary[key], value);
			}
		}

		public bool Contains(object key)
		{
			return this.eventDictionary.ContainsKey(key);
		}

		/// <summary>
		/// Umc.Core<br/>
		/// EventHandlerDictionary 의 인덱서
		/// </summary>
		/// <param name="key"></param>
		/// <returns></returns>
		public Delegate this[object key]
		{
			get { return eventDictionary[key]; }
			set { eventDictionary[key] = value; }
		}

		#region IDisposable 멤버
		/// <summary>
		/// Umc.Core<br/>
		/// EventHandlerDictionary 의 자원을 해제한다.
		/// </summary>
		public void Dispose()
		{
			if( eventDictionary != null )
				eventDictionary.Clear();

			eventDictionary = null;
		}
		#endregion
	}
}
